/****************************************************************************
** Meta object code from reading C++ file 'seatadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "seatadaptor.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'seatadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11SeatAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto SeatAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN11SeatAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SeatAdaptor",
        "D-Bus Interface",
        "org.freedesktop.DisplayManager.Seat",
        "D-Bus Introspection",
        "  <interface name=\"org.freedesktop.DisplayManager.Seat\">\n    <m"
        "ethod name=\"SwitchToGreeter\"/>\n    <method name=\"SwitchToUser\""
        ">\n      <arg direction=\"in\" type=\"s\" name=\"username\"/>\n   "
        "   <arg direction=\"in\" type=\"s\" name=\"session_name\"/>\n    <"
        "/method>\n    <method name=\"SwitchToGuest\">\n      <arg directio"
        "n=\"in\" type=\"s\" name=\"session_name\"/>\n    </method>\n    <m"
        "ethod name=\"Lock\"/>\n    <property access=\"read\" type=\"b\" na"
        "me=\"CanSwitch\"/>\n    <property access=\"read\" type=\"b\" name="
        "\"HasGuestAccount\"/>\n    <property access=\"read\" type=\"ao\" n"
        "ame=\"Sessions\"/>\n  </interface>\n",
        "Lock",
        "",
        "SwitchToGreeter",
        "SwitchToGuest",
        "session_name",
        "SwitchToUser",
        "username",
        "CanSwitch",
        "HasGuestAccount",
        "Sessions",
        "QList<QDBusObjectPath>"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'Lock'
        QtMocHelpers::SlotData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SwitchToGreeter'
        QtMocHelpers::SlotData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SwitchToGuest'
        QtMocHelpers::SlotData<void(const QString &)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'SwitchToUser'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'CanSwitch'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'HasGuestAccount'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'Sessions'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<SeatAdaptor, qt_meta_tag_ZN11SeatAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject SeatAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11SeatAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11SeatAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11SeatAdaptorE_t>.metaTypes,
    nullptr
} };

void SeatAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SeatAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Lock(); break;
        case 1: _t->SwitchToGreeter(); break;
        case 2: _t->SwitchToGuest((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->SwitchToUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->canSwitch(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->hasGuestAccount(); break;
        case 2: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->sessions(); break;
        default: break;
        }
    }
}

const QMetaObject *SeatAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SeatAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11SeatAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int SeatAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
