/* Generated by wayland-scanner 1.25.0 */

/*
 * SPDX-FileCopyrightText: 2025 UnionTech Software Technology Co., Ltd.
 * SPDX-License-Identifier: MIT
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface wl_callback_interface;

static const struct wl_interface *treeland_ddm_v1_types[] = {
	NULL,
	&wl_callback_interface,
};

static const struct wl_message treeland_ddm_v1_requests[] = {
	{ "switch_to_greeter", "", treeland_ddm_v1_types + 0 },
	{ "switch_to_user", "s", treeland_ddm_v1_types + 0 },
	{ "activate_session", "", treeland_ddm_v1_types + 0 },
	{ "deactivate_session", "", treeland_ddm_v1_types + 0 },
	{ "enable_render", "", treeland_ddm_v1_types + 0 },
	{ "disable_render", "n", treeland_ddm_v1_types + 1 },
	{ "destroy", "2", treeland_ddm_v1_types + 0 },
};

static const struct wl_message treeland_ddm_v1_events[] = {
	{ "switch_to_vt", "i", treeland_ddm_v1_types + 0 },
	{ "acquire_vt", "i", treeland_ddm_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_ddm_v1_interface = {
	"treeland_ddm_v1", 2,
	7, treeland_ddm_v1_requests,
	2, treeland_ddm_v1_events,
};

