/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.login1.User.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef LOGIN1USER_H
#define LOGIN1USER_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include "LogindDBusTypes.h"

/*
 * Proxy class for interface org.freedesktop.login1.User
 */
class OrgFreedesktopLogin1UserInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.login1.User"; }

public:
    OrgFreedesktopLogin1UserInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgFreedesktopLogin1UserInterface();

    Q_PROPERTY(NamedSessionPath Display READ display)
    inline NamedSessionPath display() const
    { return qvariant_cast< NamedSessionPath >(property("Display")); }

    Q_PROPERTY(uint GID READ gID)
    inline uint gID() const
    { return qvariant_cast< uint >(property("GID")); }

    Q_PROPERTY(bool IdleHint READ idleHint)
    inline bool idleHint() const
    { return qvariant_cast< bool >(property("IdleHint")); }

    Q_PROPERTY(qulonglong IdleSinceHint READ idleSinceHint)
    inline qulonglong idleSinceHint() const
    { return qvariant_cast< qulonglong >(property("IdleSinceHint")); }

    Q_PROPERTY(qulonglong IdleSinceHintMonotonic READ idleSinceHintMonotonic)
    inline qulonglong idleSinceHintMonotonic() const
    { return qvariant_cast< qulonglong >(property("IdleSinceHintMonotonic")); }

    Q_PROPERTY(bool Linger READ linger)
    inline bool linger() const
    { return qvariant_cast< bool >(property("Linger")); }

    Q_PROPERTY(QString Name READ name)
    inline QString name() const
    { return qvariant_cast< QString >(property("Name")); }

    Q_PROPERTY(QString RuntimePath READ runtimePath)
    inline QString runtimePath() const
    { return qvariant_cast< QString >(property("RuntimePath")); }

    Q_PROPERTY(QString Service READ service)
    inline QString service() const
    { return qvariant_cast< QString >(property("Service")); }

    Q_PROPERTY(QList<NamedSessionPath> Sessions READ sessions)
    inline QList<NamedSessionPath> sessions() const
    { return qvariant_cast< QList<NamedSessionPath> >(property("Sessions")); }

    Q_PROPERTY(QString Slice READ slice)
    inline QString slice() const
    { return qvariant_cast< QString >(property("Slice")); }

    Q_PROPERTY(QString State READ state)
    inline QString state() const
    { return qvariant_cast< QString >(property("State")); }

    Q_PROPERTY(qulonglong Timestamp READ timestamp)
    inline qulonglong timestamp() const
    { return qvariant_cast< qulonglong >(property("Timestamp")); }

    Q_PROPERTY(qulonglong TimestampMonotonic READ timestampMonotonic)
    inline qulonglong timestampMonotonic() const
    { return qvariant_cast< qulonglong >(property("TimestampMonotonic")); }

    Q_PROPERTY(uint UID READ uID)
    inline uint uID() const
    { return qvariant_cast< uint >(property("UID")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> Kill(int in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("Kill"), argumentList);
    }

    inline QDBusPendingReply<> Terminate()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Terminate"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace freedesktop {
    namespace login1 {
      using User = ::OrgFreedesktopLogin1UserInterface;
    }
  }
}

#endif
