/****************************************************************************
** Meta object code from reading C++ file 'Login1Manager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "Login1Manager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Login1Manager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN36OrgFreedesktopLogin1ManagerInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopLogin1ManagerInterface::qt_create_metaobjectdata<qt_meta_tag_ZN36OrgFreedesktopLogin1ManagerInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopLogin1ManagerInterface",
        "PrepareForShutdown",
        "",
        "in0",
        "PrepareForSleep",
        "SeatNew",
        "QDBusObjectPath",
        "in1",
        "SeatRemoved",
        "SessionNew",
        "SessionRemoved",
        "UserNew",
        "UserRemoved",
        "ActivateSession",
        "QDBusPendingReply<>",
        "ActivateSessionOnSeat",
        "AttachDevice",
        "in2",
        "CanHibernate",
        "QDBusPendingReply<QString>",
        "CanHybridSleep",
        "CanPowerOff",
        "CanReboot",
        "CanSuspend",
        "FlushDevices",
        "GetSeat",
        "QDBusPendingReply<QDBusObjectPath>",
        "GetSession",
        "GetSessionByPID",
        "GetUser",
        "GetUserByPID",
        "Hibernate",
        "HybridSleep",
        "Inhibit",
        "QDBusPendingReply<QDBusUnixFileDescriptor>",
        "in3",
        "KillSession",
        "KillUser",
        "ListInhibitors",
        "QDBusPendingReply<InhibitorList>",
        "ListSeats",
        "QDBusPendingReply<NamedSeatPathList>",
        "ListSessions",
        "QDBusPendingReply<SessionInfoList>",
        "ListUsers",
        "QDBusPendingReply<UserInfoList>",
        "LockSession",
        "LockSessions",
        "PowerOff",
        "Reboot",
        "ReleaseSession",
        "SetUserLinger",
        "Suspend",
        "TerminateSeat",
        "TerminateSession",
        "TerminateUser",
        "UnlockSession",
        "UnlockSessions",
        "BlockInhibited",
        "DelayInhibited",
        "HandleHibernateKey",
        "HandleLidSwitch",
        "HandlePowerKey",
        "HandleSuspendKey",
        "IdleAction",
        "IdleActionUSec",
        "IdleHint",
        "IdleSinceHint",
        "IdleSinceHintMonotonic",
        "InhibitDelayMaxUSec",
        "KillExcludeUsers",
        "KillOnlyUsers",
        "KillUserProcesses",
        "NAutoVTs",
        "PreparingForShutdown",
        "PreparingForSleep"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PrepareForShutdown'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'PrepareForSleep'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'SeatNew'
        QtMocHelpers::SignalData<void(const QString &, const QDBusObjectPath &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'SeatRemoved'
        QtMocHelpers::SignalData<void(const QString &, const QDBusObjectPath &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'SessionNew'
        QtMocHelpers::SignalData<void(const QString &, const QDBusObjectPath &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'SessionRemoved'
        QtMocHelpers::SignalData<void(const QString &, const QDBusObjectPath &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'UserNew'
        QtMocHelpers::SignalData<void(uint, const QDBusObjectPath &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'UserRemoved'
        QtMocHelpers::SignalData<void(uint, const QDBusObjectPath &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'ActivateSession'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'ActivateSessionOnSeat'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, const QString &)>(15, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'AttachDevice'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, const QString &, bool)>(16, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 7 }, { QMetaType::Bool, 17 },
        }}),
        // Slot 'CanHibernate'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(18, 2, QMC::AccessPublic, 0x80000000 | 19),
        // Slot 'CanHybridSleep'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(20, 2, QMC::AccessPublic, 0x80000000 | 19),
        // Slot 'CanPowerOff'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(21, 2, QMC::AccessPublic, 0x80000000 | 19),
        // Slot 'CanReboot'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(22, 2, QMC::AccessPublic, 0x80000000 | 19),
        // Slot 'CanSuspend'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(23, 2, QMC::AccessPublic, 0x80000000 | 19),
        // Slot 'FlushDevices'
        QtMocHelpers::SlotData<QDBusPendingReply<>(bool)>(24, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'GetSeat'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QString &)>(25, 2, QMC::AccessPublic, 0x80000000 | 26, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'GetSession'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QString &)>(27, 2, QMC::AccessPublic, 0x80000000 | 26, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'GetSessionByPID'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(uint)>(28, 2, QMC::AccessPublic, 0x80000000 | 26, {{
            { QMetaType::UInt, 3 },
        }}),
        // Slot 'GetUser'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(uint)>(29, 2, QMC::AccessPublic, 0x80000000 | 26, {{
            { QMetaType::UInt, 3 },
        }}),
        // Slot 'GetUserByPID'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(uint)>(30, 2, QMC::AccessPublic, 0x80000000 | 26, {{
            { QMetaType::UInt, 3 },
        }}),
        // Slot 'Hibernate'
        QtMocHelpers::SlotData<QDBusPendingReply<>(bool)>(31, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'HybridSleep'
        QtMocHelpers::SlotData<QDBusPendingReply<>(bool)>(32, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'Inhibit'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusUnixFileDescriptor>(const QString &, const QString &, const QString &, const QString &)>(33, 2, QMC::AccessPublic, 0x80000000 | 34, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 7 }, { QMetaType::QString, 17 }, { QMetaType::QString, 35 },
        }}),
        // Slot 'KillSession'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, const QString &, int)>(36, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 7 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'KillUser'
        QtMocHelpers::SlotData<QDBusPendingReply<>(uint, int)>(37, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::UInt, 3 }, { QMetaType::Int, 7 },
        }}),
        // Slot 'ListInhibitors'
        QtMocHelpers::SlotData<QDBusPendingReply<InhibitorList>()>(38, 2, QMC::AccessPublic, 0x80000000 | 39),
        // Slot 'ListSeats'
        QtMocHelpers::SlotData<QDBusPendingReply<NamedSeatPathList>()>(40, 2, QMC::AccessPublic, 0x80000000 | 41),
        // Slot 'ListSessions'
        QtMocHelpers::SlotData<QDBusPendingReply<SessionInfoList>()>(42, 2, QMC::AccessPublic, 0x80000000 | 43),
        // Slot 'ListUsers'
        QtMocHelpers::SlotData<QDBusPendingReply<UserInfoList>()>(44, 2, QMC::AccessPublic, 0x80000000 | 45),
        // Slot 'LockSession'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(46, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'LockSessions'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(47, 2, QMC::AccessPublic, 0x80000000 | 14),
        // Slot 'PowerOff'
        QtMocHelpers::SlotData<QDBusPendingReply<>(bool)>(48, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'Reboot'
        QtMocHelpers::SlotData<QDBusPendingReply<>(bool)>(49, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'ReleaseSession'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(50, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'SetUserLinger'
        QtMocHelpers::SlotData<QDBusPendingReply<>(uint, bool, bool)>(51, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::UInt, 3 }, { QMetaType::Bool, 7 }, { QMetaType::Bool, 17 },
        }}),
        // Slot 'Suspend'
        QtMocHelpers::SlotData<QDBusPendingReply<>(bool)>(52, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'TerminateSeat'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(53, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'TerminateSession'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(54, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'TerminateUser'
        QtMocHelpers::SlotData<QDBusPendingReply<>(uint)>(55, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::UInt, 3 },
        }}),
        // Slot 'UnlockSession'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(56, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'UnlockSessions'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(57, 2, QMC::AccessPublic, 0x80000000 | 14),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'BlockInhibited'
        QtMocHelpers::PropertyData<QString>(58, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'DelayInhibited'
        QtMocHelpers::PropertyData<QString>(59, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'HandleHibernateKey'
        QtMocHelpers::PropertyData<QString>(60, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'HandleLidSwitch'
        QtMocHelpers::PropertyData<QString>(61, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'HandlePowerKey'
        QtMocHelpers::PropertyData<QString>(62, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'HandleSuspendKey'
        QtMocHelpers::PropertyData<QString>(63, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'IdleAction'
        QtMocHelpers::PropertyData<QString>(64, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'IdleActionUSec'
        QtMocHelpers::PropertyData<qulonglong>(65, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'IdleHint'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'IdleSinceHint'
        QtMocHelpers::PropertyData<qulonglong>(67, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'IdleSinceHintMonotonic'
        QtMocHelpers::PropertyData<qulonglong>(68, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'InhibitDelayMaxUSec'
        QtMocHelpers::PropertyData<qulonglong>(69, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'KillExcludeUsers'
        QtMocHelpers::PropertyData<QStringList>(70, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'KillOnlyUsers'
        QtMocHelpers::PropertyData<QStringList>(71, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'KillUserProcesses'
        QtMocHelpers::PropertyData<bool>(72, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'NAutoVTs'
        QtMocHelpers::PropertyData<uint>(73, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'PreparingForShutdown'
        QtMocHelpers::PropertyData<bool>(74, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'PreparingForSleep'
        QtMocHelpers::PropertyData<bool>(75, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopLogin1ManagerInterface, qt_meta_tag_ZN36OrgFreedesktopLogin1ManagerInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopLogin1ManagerInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36OrgFreedesktopLogin1ManagerInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36OrgFreedesktopLogin1ManagerInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN36OrgFreedesktopLogin1ManagerInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopLogin1ManagerInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopLogin1ManagerInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PrepareForShutdown((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->PrepareForSleep((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->SeatNew((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 3: _t->SeatRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 4: _t->SessionNew((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 5: _t->SessionRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 6: _t->UserNew((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 7: _t->UserRemoved((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 8: { QDBusPendingReply<> _r = _t->ActivateSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->ActivateSessionOnSeat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->AttachDevice((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<QString> _r = _t->CanHibernate();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<QString> _r = _t->CanHybridSleep();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<QString> _r = _t->CanPowerOff();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<QString> _r = _t->CanReboot();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<QString> _r = _t->CanSuspend();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QDBusPendingReply<> _r = _t->FlushDevices((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 17: { QDBusPendingReply<QDBusObjectPath> _r = _t->GetSeat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 18: { QDBusPendingReply<QDBusObjectPath> _r = _t->GetSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QDBusPendingReply<QDBusObjectPath> _r = _t->GetSessionByPID((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 20: { QDBusPendingReply<QDBusObjectPath> _r = _t->GetUser((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 21: { QDBusPendingReply<QDBusObjectPath> _r = _t->GetUserByPID((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 22: { QDBusPendingReply<> _r = _t->Hibernate((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 23: { QDBusPendingReply<> _r = _t->HybridSleep((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 24: { QDBusPendingReply<QDBusUnixFileDescriptor> _r = _t->Inhibit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusUnixFileDescriptor>*>(_a[0]) = std::move(_r); }  break;
        case 25: { QDBusPendingReply<> _r = _t->KillSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 26: { QDBusPendingReply<> _r = _t->KillUser((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 27: { QDBusPendingReply<InhibitorList> _r = _t->ListInhibitors();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<InhibitorList>*>(_a[0]) = std::move(_r); }  break;
        case 28: { QDBusPendingReply<NamedSeatPathList> _r = _t->ListSeats();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<NamedSeatPathList>*>(_a[0]) = std::move(_r); }  break;
        case 29: { QDBusPendingReply<SessionInfoList> _r = _t->ListSessions();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<SessionInfoList>*>(_a[0]) = std::move(_r); }  break;
        case 30: { QDBusPendingReply<UserInfoList> _r = _t->ListUsers();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<UserInfoList>*>(_a[0]) = std::move(_r); }  break;
        case 31: { QDBusPendingReply<> _r = _t->LockSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 32: { QDBusPendingReply<> _r = _t->LockSessions();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 33: { QDBusPendingReply<> _r = _t->PowerOff((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 34: { QDBusPendingReply<> _r = _t->Reboot((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 35: { QDBusPendingReply<> _r = _t->ReleaseSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 36: { QDBusPendingReply<> _r = _t->SetUserLinger((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 37: { QDBusPendingReply<> _r = _t->Suspend((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 38: { QDBusPendingReply<> _r = _t->TerminateSeat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 39: { QDBusPendingReply<> _r = _t->TerminateSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 40: { QDBusPendingReply<> _r = _t->TerminateUser((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 41: { QDBusPendingReply<> _r = _t->UnlockSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 42: { QDBusPendingReply<> _r = _t->UnlockSessions();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1ManagerInterface::*)(bool )>(_a, &OrgFreedesktopLogin1ManagerInterface::PrepareForShutdown, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1ManagerInterface::*)(bool )>(_a, &OrgFreedesktopLogin1ManagerInterface::PrepareForSleep, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1ManagerInterface::*)(const QString & , const QDBusObjectPath & )>(_a, &OrgFreedesktopLogin1ManagerInterface::SeatNew, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1ManagerInterface::*)(const QString & , const QDBusObjectPath & )>(_a, &OrgFreedesktopLogin1ManagerInterface::SeatRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1ManagerInterface::*)(const QString & , const QDBusObjectPath & )>(_a, &OrgFreedesktopLogin1ManagerInterface::SessionNew, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1ManagerInterface::*)(const QString & , const QDBusObjectPath & )>(_a, &OrgFreedesktopLogin1ManagerInterface::SessionRemoved, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1ManagerInterface::*)(uint , const QDBusObjectPath & )>(_a, &OrgFreedesktopLogin1ManagerInterface::UserNew, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1ManagerInterface::*)(uint , const QDBusObjectPath & )>(_a, &OrgFreedesktopLogin1ManagerInterface::UserRemoved, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->blockInhibited(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->delayInhibited(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->handleHibernateKey(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->handleLidSwitch(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->handlePowerKey(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->handleSuspendKey(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->idleAction(); break;
        case 7: *reinterpret_cast<qulonglong*>(_v) = _t->idleActionUSec(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->idleHint(); break;
        case 9: *reinterpret_cast<qulonglong*>(_v) = _t->idleSinceHint(); break;
        case 10: *reinterpret_cast<qulonglong*>(_v) = _t->idleSinceHintMonotonic(); break;
        case 11: *reinterpret_cast<qulonglong*>(_v) = _t->inhibitDelayMaxUSec(); break;
        case 12: *reinterpret_cast<QStringList*>(_v) = _t->killExcludeUsers(); break;
        case 13: *reinterpret_cast<QStringList*>(_v) = _t->killOnlyUsers(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->killUserProcesses(); break;
        case 15: *reinterpret_cast<uint*>(_v) = _t->nAutoVTs(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->preparingForShutdown(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->preparingForSleep(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgFreedesktopLogin1ManagerInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopLogin1ManagerInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36OrgFreedesktopLogin1ManagerInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopLogin1ManagerInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 43)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 43;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopLogin1ManagerInterface::PrepareForShutdown(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OrgFreedesktopLogin1ManagerInterface::PrepareForSleep(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OrgFreedesktopLogin1ManagerInterface::SeatNew(const QString & _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OrgFreedesktopLogin1ManagerInterface::SeatRemoved(const QString & _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void OrgFreedesktopLogin1ManagerInterface::SessionNew(const QString & _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void OrgFreedesktopLogin1ManagerInterface::SessionRemoved(const QString & _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void OrgFreedesktopLogin1ManagerInterface::UserNew(uint _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void OrgFreedesktopLogin1ManagerInterface::UserRemoved(uint _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}
QT_WARNING_POP
