/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.DisplayManager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "displaymanageradaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class DisplayManagerAdaptor
 */

DisplayManagerAdaptor::DisplayManagerAdaptor(DDM::DisplayManager *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

DisplayManagerAdaptor::~DisplayManagerAdaptor()
{
    // destructor
}

QDBusObjectPath DisplayManagerAdaptor::lastSession() const
{
    // get the value of property LastSession
    return qvariant_cast< QDBusObjectPath >(parent()->property("LastSession"));
}

QList<QDBusObjectPath> DisplayManagerAdaptor::seats() const
{
    // get the value of property Seats
    return qvariant_cast< QList<QDBusObjectPath> >(parent()->property("Seats"));
}

QList<QDBusObjectPath> DisplayManagerAdaptor::sessions() const
{
    // get the value of property Sessions
    return qvariant_cast< QList<QDBusObjectPath> >(parent()->property("Sessions"));
}


#include "moc_displaymanageradaptor.cpp"
