/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.DisplayManager.Seat.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef SEATADAPTOR_H
#define SEATADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "DisplayManager.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.freedesktop.DisplayManager.Seat
 */
class SeatAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.freedesktop.DisplayManager.Seat")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.freedesktop.DisplayManager.Seat\">\n"
"    <method name=\"SwitchToGreeter\"/>\n"
"    <method name=\"SwitchToUser\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"username\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"session_name\"/>\n"
"    </method>\n"
"    <method name=\"SwitchToGuest\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"session_name\"/>\n"
"    </method>\n"
"    <method name=\"Lock\"/>\n"
"    <property access=\"read\" type=\"b\" name=\"CanSwitch\"/>\n"
"    <property access=\"read\" type=\"b\" name=\"HasGuestAccount\"/>\n"
"    <property access=\"read\" type=\"ao\" name=\"Sessions\"/>\n"
"  </interface>\n"
        "")
public:
    SeatAdaptor(DDM::DisplayManagerSeat *parent);
    ~SeatAdaptor() override;

    inline DDM::DisplayManagerSeat *parent() const
    { return static_cast<DDM::DisplayManagerSeat *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(bool CanSwitch READ canSwitch)
    bool canSwitch() const;

    Q_PROPERTY(bool HasGuestAccount READ hasGuestAccount)
    bool hasGuestAccount() const;

    Q_PROPERTY(QList<QDBusObjectPath> Sessions READ sessions)
    QList<QDBusObjectPath> sessions() const;

public Q_SLOTS: // METHODS
    void Lock();
    void SwitchToGreeter();
    void SwitchToGuest(const QString &session_name);
    void SwitchToUser(const QString &username, const QString &session_name);
Q_SIGNALS: // SIGNALS
};

#endif
