/****************************************************************************
** Meta object code from reading C++ file 'Display.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../Display.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Display.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3DDM7DisplayE_t {};
} // unnamed namespace

template <> constexpr inline auto DDM::Display::qt_create_metaobjectdata<qt_meta_tag_ZN3DDM7DisplayE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DDM::Display",
        "stopped",
        "",
        "loginFailed",
        "QLocalSocket*",
        "socket",
        "user",
        "start",
        "stop",
        "connected",
        "login",
        "password",
        "Session",
        "session",
        "logout",
        "id",
        "lock",
        "unlock",
        "DisplayServerType",
        "X11",
        "Wayland",
        "Treeland"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stopped'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loginFailed'
        QtMocHelpers::SignalData<void(QLocalSocket *, const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'start'
        QtMocHelpers::SlotData<bool()>(7, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'connected'
        QtMocHelpers::SlotData<void(QLocalSocket *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'login'
        QtMocHelpers::SlotData<void(QLocalSocket *, const QString &, const QString &, const Session &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 }, { QMetaType::QString, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'logout'
        QtMocHelpers::SlotData<void(QLocalSocket *, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Int, 15 },
        }}),
        // Slot 'lock'
        QtMocHelpers::SlotData<void(QLocalSocket *, int)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Int, 15 },
        }}),
        // Slot 'unlock'
        QtMocHelpers::SlotData<void(QLocalSocket *, const QString &, const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 }, { QMetaType::QString, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DisplayServerType'
        QtMocHelpers::EnumData<enum DisplayServerType>(18, 18, QMC::EnumFlags{}).add({
            {   19, DisplayServerType::X11 },
            {   20, DisplayServerType::Wayland },
            {   21, DisplayServerType::Treeland },
        }),
    };
    return QtMocHelpers::metaObjectData<Display, qt_meta_tag_ZN3DDM7DisplayE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DDM::Display::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM7DisplayE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM7DisplayE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3DDM7DisplayE_t>.metaTypes,
    nullptr
} };

void DDM::Display::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Display *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stopped(); break;
        case 1: _t->loginFailed((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: { bool _r = _t->start();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->stop(); break;
        case 4: _t->connected((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1]))); break;
        case 5: _t->login((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Session>>(_a[4]))); break;
        case 6: _t->logout((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 7: _t->lock((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->unlock((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Display::*)()>(_a, &Display::stopped, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Display::*)(QLocalSocket * , const QString & )>(_a, &Display::loginFailed, 1))
            return;
    }
}

const QMetaObject *DDM::Display::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DDM::Display::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM7DisplayE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DDM::Display::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void DDM::Display::stopped()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DDM::Display::loginFailed(QLocalSocket * _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
