/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.login1.Session.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef LOGIN1SESSION_H
#define LOGIN1SESSION_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include "LogindDBusTypes.h"

/*
 * Proxy class for interface org.freedesktop.login1.Session
 */
class OrgFreedesktopLogin1SessionInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.login1.Session"; }

public:
    OrgFreedesktopLogin1SessionInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgFreedesktopLogin1SessionInterface();

    Q_PROPERTY(bool Active READ active)
    inline bool active() const
    { return qvariant_cast< bool >(property("Active")); }

    Q_PROPERTY(uint Audit READ audit)
    inline uint audit() const
    { return qvariant_cast< uint >(property("Audit")); }

    Q_PROPERTY(QString Class READ className)
    inline QString className() const
    { return qvariant_cast< QString >(property("Class")); }

    Q_PROPERTY(QString Desktop READ desktop)
    inline QString desktop() const
    { return qvariant_cast< QString >(property("Desktop")); }

    Q_PROPERTY(QString Display READ display)
    inline QString display() const
    { return qvariant_cast< QString >(property("Display")); }

    Q_PROPERTY(QString Id READ id)
    inline QString id() const
    { return qvariant_cast< QString >(property("Id")); }

    Q_PROPERTY(bool IdleHint READ idleHint)
    inline bool idleHint() const
    { return qvariant_cast< bool >(property("IdleHint")); }

    Q_PROPERTY(qulonglong IdleSinceHint READ idleSinceHint)
    inline qulonglong idleSinceHint() const
    { return qvariant_cast< qulonglong >(property("IdleSinceHint")); }

    Q_PROPERTY(qulonglong IdleSinceHintMonotonic READ idleSinceHintMonotonic)
    inline qulonglong idleSinceHintMonotonic() const
    { return qvariant_cast< qulonglong >(property("IdleSinceHintMonotonic")); }

    Q_PROPERTY(uint Leader READ leader)
    inline uint leader() const
    { return qvariant_cast< uint >(property("Leader")); }

    Q_PROPERTY(QString Name READ name)
    inline QString name() const
    { return qvariant_cast< QString >(property("Name")); }

    Q_PROPERTY(bool Remote READ remote)
    inline bool remote() const
    { return qvariant_cast< bool >(property("Remote")); }

    Q_PROPERTY(QString RemoteHost READ remoteHost)
    inline QString remoteHost() const
    { return qvariant_cast< QString >(property("RemoteHost")); }

    Q_PROPERTY(QString RemoteUser READ remoteUser)
    inline QString remoteUser() const
    { return qvariant_cast< QString >(property("RemoteUser")); }

    Q_PROPERTY(QString Scope READ scope)
    inline QString scope() const
    { return qvariant_cast< QString >(property("Scope")); }

    Q_PROPERTY(NamedSeatPath Seat READ seat)
    inline NamedSeatPath seat() const
    { return qvariant_cast< NamedSeatPath >(property("Seat")); }

    Q_PROPERTY(QString Service READ service)
    inline QString service() const
    { return qvariant_cast< QString >(property("Service")); }

    Q_PROPERTY(QString State READ state)
    inline QString state() const
    { return qvariant_cast< QString >(property("State")); }

    Q_PROPERTY(QString TTY READ tTY)
    inline QString tTY() const
    { return qvariant_cast< QString >(property("TTY")); }

    Q_PROPERTY(qulonglong Timestamp READ timestamp)
    inline qulonglong timestamp() const
    { return qvariant_cast< qulonglong >(property("Timestamp")); }

    Q_PROPERTY(qulonglong TimestampMonotonic READ timestampMonotonic)
    inline qulonglong timestampMonotonic() const
    { return qvariant_cast< qulonglong >(property("TimestampMonotonic")); }

    Q_PROPERTY(QString Type READ type)
    inline QString type() const
    { return qvariant_cast< QString >(property("Type")); }

    Q_PROPERTY(NamedUserPath User READ user)
    inline NamedUserPath user() const
    { return qvariant_cast< NamedUserPath >(property("User")); }

    Q_PROPERTY(uint VTNr READ vTNr)
    inline uint vTNr() const
    { return qvariant_cast< uint >(property("VTNr")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> Activate()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Activate"), argumentList);
    }

    inline QDBusPendingReply<> Kill(const QString &in0, int in1)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0) << QVariant::fromValue(in1);
        return asyncCallWithArgumentList(QStringLiteral("Kill"), argumentList);
    }

    inline QDBusPendingReply<> requestLock()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Lock"), argumentList);
    }

    inline QDBusPendingReply<> PauseDeviceComplete(uint in0, uint in1)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0) << QVariant::fromValue(in1);
        return asyncCallWithArgumentList(QStringLiteral("PauseDeviceComplete"), argumentList);
    }

    inline QDBusPendingReply<> ReleaseControl()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("ReleaseControl"), argumentList);
    }

    inline QDBusPendingReply<> ReleaseDevice(uint in0, uint in1)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0) << QVariant::fromValue(in1);
        return asyncCallWithArgumentList(QStringLiteral("ReleaseDevice"), argumentList);
    }

    inline QDBusPendingReply<> SetIdleHint(bool in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("SetIdleHint"), argumentList);
    }

    inline QDBusPendingReply<> TakeControl(bool in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("TakeControl"), argumentList);
    }

    inline QDBusPendingReply<QDBusUnixFileDescriptor, bool> TakeDevice(uint in0, uint in1)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0) << QVariant::fromValue(in1);
        return asyncCallWithArgumentList(QStringLiteral("TakeDevice"), argumentList);
    }
    inline QDBusReply<QDBusUnixFileDescriptor> TakeDevice(uint in0, uint in1, bool &out1)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0) << QVariant::fromValue(in1);
        QDBusMessage reply = callWithArgumentList(QDBus::Block, QStringLiteral("TakeDevice"), argumentList);
        if (reply.type() == QDBusMessage::ReplyMessage && reply.arguments().size() == 2) {
            out1 = qdbus_cast<bool>(reply.arguments().at(1));
        }
        return reply;
    }

    inline QDBusPendingReply<> Terminate()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Terminate"), argumentList);
    }

    inline QDBusPendingReply<> requestUnlock()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Unlock"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void Lock();
    void PauseDevice(uint in0, uint in1, const QString &in2);
    void ResumeDevice(uint in0, uint in1, const QDBusUnixFileDescriptor &in2);
    void Unlock();
};

namespace org {
  namespace freedesktop {
    namespace login1 {
      using Session = ::OrgFreedesktopLogin1SessionInterface;
    }
  }
}

#endif
