/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.DisplayManager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "dbus_ddm_displaymanager.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class DDMDisplayManagerAdaptor
 */

DDMDisplayManagerAdaptor::DDMDisplayManagerAdaptor(DDM::DisplayManager *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

DDMDisplayManagerAdaptor::~DDMDisplayManagerAdaptor()
{
    // destructor
}

QString DDMDisplayManagerAdaptor::AuthInfo()
{
    // handle method call org.deepin.DisplayManager.AuthInfo
    return parent()->AuthInfo();
}

QString DDMDisplayManagerAdaptor::LastActivatedUser()
{
    // handle method call org.deepin.DisplayManager.LastActivatedUser
    return parent()->LastActivatedUser();
}


#include "moc_dbus_ddm_displaymanager.cpp"
