/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.DisplayManager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef DISPLAYMANAGERADAPTOR_H
#define DISPLAYMANAGERADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "DisplayManager.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.freedesktop.DisplayManager
 */
class DisplayManagerAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.freedesktop.DisplayManager")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.freedesktop.DisplayManager\">\n"
"    <signal name=\"SeatAdded\">\n"
"      <arg type=\"o\" name=\"seat\"/>\n"
"    </signal>\n"
"    <signal name=\"SeatRemoved\">\n"
"      <arg type=\"o\" name=\"seat\"/>\n"
"    </signal>\n"
"    <signal name=\"SessionAdded\">\n"
"      <arg type=\"o\" name=\"session\"/>\n"
"    </signal>\n"
"    <signal name=\"SessionRemoved\">\n"
"      <arg type=\"o\" name=\"session\"/>\n"
"    </signal>\n"
"    <property access=\"read\" type=\"ao\" name=\"Seats\"/>\n"
"    <property access=\"read\" type=\"ao\" name=\"Sessions\"/>\n"
"    <property access=\"read\" type=\"o\" name=\"LastSession\"/>\n"
"  </interface>\n"
        "")
public:
    DisplayManagerAdaptor(DDM::DisplayManager *parent);
    ~DisplayManagerAdaptor() override;

    inline DDM::DisplayManager *parent() const
    { return static_cast<DDM::DisplayManager *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(QDBusObjectPath LastSession READ lastSession)
    QDBusObjectPath lastSession() const;

    Q_PROPERTY(QList<QDBusObjectPath> Seats READ seats)
    QList<QDBusObjectPath> seats() const;

    Q_PROPERTY(QList<QDBusObjectPath> Sessions READ sessions)
    QList<QDBusObjectPath> sessions() const;

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
    void SeatAdded(const QDBusObjectPath &seat);
    void SeatRemoved(const QDBusObjectPath &seat);
    void SessionAdded(const QDBusObjectPath &session);
    void SessionRemoved(const QDBusObjectPath &session);
};

#endif
