/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.DisplayManager.Seat.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "seatadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class SeatAdaptor
 */

SeatAdaptor::SeatAdaptor(DDM::DisplayManagerSeat *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

SeatAdaptor::~SeatAdaptor()
{
    // destructor
}

bool SeatAdaptor::canSwitch() const
{
    // get the value of property CanSwitch
    return qvariant_cast< bool >(parent()->property("CanSwitch"));
}

bool SeatAdaptor::hasGuestAccount() const
{
    // get the value of property HasGuestAccount
    return qvariant_cast< bool >(parent()->property("HasGuestAccount"));
}

QList<QDBusObjectPath> SeatAdaptor::sessions() const
{
    // get the value of property Sessions
    return qvariant_cast< QList<QDBusObjectPath> >(parent()->property("Sessions"));
}

void SeatAdaptor::Lock()
{
    // handle method call org.freedesktop.DisplayManager.Seat.Lock
    parent()->Lock();
}

void SeatAdaptor::SwitchToGreeter()
{
    // handle method call org.freedesktop.DisplayManager.Seat.SwitchToGreeter
    parent()->SwitchToGreeter();
}

void SeatAdaptor::SwitchToGuest(const QString &session_name)
{
    // handle method call org.freedesktop.DisplayManager.Seat.SwitchToGuest
    parent()->SwitchToGuest(session_name);
}

void SeatAdaptor::SwitchToUser(const QString &username, const QString &session_name)
{
    // handle method call org.freedesktop.DisplayManager.Seat.SwitchToUser
    parent()->SwitchToUser(username, session_name);
}


#include "moc_seatadaptor.cpp"
