/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.DisplayManager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef DBUS_DDM_DISPLAYMANAGER_H
#define DBUS_DDM_DISPLAYMANAGER_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "DisplayManager.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.deepin.DisplayManager
 */
class DDMDisplayManagerAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.deepin.DisplayManager")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.deepin.DisplayManager\">\n"
"    <method name=\"AuthInfo\">\n"
"      <arg direction=\"out\" type=\"s\" name=\"auth socket\"/>\n"
"    </method>\n"
"    <signal name=\"AuthInfoChanged\"/>\n"
"    <method name=\"LastActivatedUser\">\n"
"      <arg direction=\"out\" type=\"s\" name=\"last activated user\"/>\n"
"    </method>\n"
"    <signal name=\"LastActivatedUserChanged\"/>\n"
"  </interface>\n"
        "")
public:
    DDMDisplayManagerAdaptor(DDM::DisplayManager *parent);
    ~DDMDisplayManagerAdaptor() override;

    inline DDM::DisplayManager *parent() const
    { return static_cast<DDM::DisplayManager *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QString AuthInfo();
    QString LastActivatedUser();
Q_SIGNALS: // SIGNALS
    void AuthInfoChanged();
    void LastActivatedUserChanged();
};

#endif
