/****************************************************************************
** Meta object code from reading C++ file 'DisplayManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../DisplayManager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'DisplayManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3DDM14DisplayManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto DDM::DisplayManager::qt_create_metaobjectdata<qt_meta_tag_ZN3DDM14DisplayManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DDM::DisplayManager",
        "SeatAdded",
        "",
        "ObjectPath",
        "seat",
        "SeatRemoved",
        "SessionAdded",
        "session",
        "SessionRemoved",
        "LastSessionChanged",
        "AuthInfoChanged",
        "LastActivatedUserChanged",
        "AddSeat",
        "name",
        "RemoveSeat",
        "AddSession",
        "user",
        "vtnr",
        "RemoveSession",
        "setLastSession",
        "setAuthInfo",
        "authSocket",
        "setLastActivatedUser",
        "lastActivatedUser",
        "Seats",
        "QList<QDBusObjectPath>",
        "Sessions",
        "LastSession",
        "QDBusObjectPath",
        "AuthInfo",
        "LastActivatedUser"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'SeatAdded'
        QtMocHelpers::SignalData<void(ObjectPath)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'SeatRemoved'
        QtMocHelpers::SignalData<void(ObjectPath)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'SessionAdded'
        QtMocHelpers::SignalData<void(ObjectPath)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 7 },
        }}),
        // Signal 'SessionRemoved'
        QtMocHelpers::SignalData<void(ObjectPath)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 7 },
        }}),
        // Signal 'LastSessionChanged'
        QtMocHelpers::SignalData<void(ObjectPath)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 7 },
        }}),
        // Signal 'AuthInfoChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'LastActivatedUserChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'AddSeat'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'RemoveSeat'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'AddSession'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const int &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { QMetaType::QString, 4 }, { QMetaType::QString, 16 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'RemoveSession'
        QtMocHelpers::SlotData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'setLastSession'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'setAuthInfo'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'setLastActivatedUser'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Seats'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'Sessions'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(26, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'LastSession'
        QtMocHelpers::PropertyData<QDBusObjectPath>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'AuthInfo'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'LastActivatedUser'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DisplayManager, qt_meta_tag_ZN3DDM14DisplayManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DDM::DisplayManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM14DisplayManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM14DisplayManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3DDM14DisplayManagerE_t>.metaTypes,
    nullptr
} };

void DDM::DisplayManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DisplayManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->SeatAdded((*reinterpret_cast<std::add_pointer_t<ObjectPath>>(_a[1]))); break;
        case 1: _t->SeatRemoved((*reinterpret_cast<std::add_pointer_t<ObjectPath>>(_a[1]))); break;
        case 2: _t->SessionAdded((*reinterpret_cast<std::add_pointer_t<ObjectPath>>(_a[1]))); break;
        case 3: _t->SessionRemoved((*reinterpret_cast<std::add_pointer_t<ObjectPath>>(_a[1]))); break;
        case 4: _t->LastSessionChanged((*reinterpret_cast<std::add_pointer_t<ObjectPath>>(_a[1]))); break;
        case 5: _t->AuthInfoChanged(); break;
        case 6: _t->LastActivatedUserChanged(); break;
        case 7: _t->AddSeat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->RemoveSeat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->AddSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 10: _t->RemoveSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->setLastSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->setAuthInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->setLastActivatedUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DisplayManager::*)(ObjectPath )>(_a, &DisplayManager::SeatAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayManager::*)(ObjectPath )>(_a, &DisplayManager::SeatRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayManager::*)(ObjectPath )>(_a, &DisplayManager::SessionAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayManager::*)(ObjectPath )>(_a, &DisplayManager::SessionRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayManager::*)(ObjectPath )>(_a, &DisplayManager::LastSessionChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayManager::*)()>(_a, &DisplayManager::AuthInfoChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayManager::*)()>(_a, &DisplayManager::LastActivatedUserChanged, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QDBusObjectPath> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->Seats(); break;
        case 1: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->Sessions(); break;
        case 2: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->LastSession(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->AuthInfo(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->LastActivatedUser(); break;
        default: break;
        }
    }
}

const QMetaObject *DDM::DisplayManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DDM::DisplayManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM14DisplayManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DDM::DisplayManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void DDM::DisplayManager::SeatAdded(ObjectPath _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DDM::DisplayManager::SeatRemoved(ObjectPath _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DDM::DisplayManager::SessionAdded(ObjectPath _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DDM::DisplayManager::SessionRemoved(ObjectPath _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DDM::DisplayManager::LastSessionChanged(ObjectPath _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void DDM::DisplayManager::AuthInfoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void DDM::DisplayManager::LastActivatedUserChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
namespace {
struct qt_meta_tag_ZN3DDM18DisplayManagerSeatE_t {};
} // unnamed namespace

template <> constexpr inline auto DDM::DisplayManagerSeat::qt_create_metaobjectdata<qt_meta_tag_ZN3DDM18DisplayManagerSeatE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DDM::DisplayManagerSeat",
        "CanSwitch",
        "HasGuestAccount",
        "Sessions",
        "QList<QDBusObjectPath>"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'CanSwitch'
        QtMocHelpers::PropertyData<bool>(1, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'HasGuestAccount'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'Sessions'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DisplayManagerSeat, qt_meta_tag_ZN3DDM18DisplayManagerSeatE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DDM::DisplayManagerSeat::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM18DisplayManagerSeatE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM18DisplayManagerSeatE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3DDM18DisplayManagerSeatE_t>.metaTypes,
    nullptr
} };

void DDM::DisplayManagerSeat::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DisplayManagerSeat *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QDBusObjectPath> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->CanSwitch(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->HasGuestAccount(); break;
        case 2: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->Sessions(); break;
        default: break;
        }
    }
}

const QMetaObject *DDM::DisplayManagerSeat::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DDM::DisplayManagerSeat::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM18DisplayManagerSeatE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int DDM::DisplayManagerSeat::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3DDM21DisplayManagerSessionE_t {};
} // unnamed namespace

template <> constexpr inline auto DDM::DisplayManagerSession::qt_create_metaobjectdata<qt_meta_tag_ZN3DDM21DisplayManagerSessionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DDM::DisplayManagerSession",
        "Seat",
        "QDBusObjectPath",
        "UserName"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Seat'
        QtMocHelpers::PropertyData<QDBusObjectPath>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'UserName'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DisplayManagerSession, qt_meta_tag_ZN3DDM21DisplayManagerSessionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DDM::DisplayManagerSession::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM21DisplayManagerSessionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM21DisplayManagerSessionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3DDM21DisplayManagerSessionE_t>.metaTypes,
    nullptr
} };

void DDM::DisplayManagerSession::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DisplayManagerSession *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->SeatPath(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->User(); break;
        default: break;
        }
    }
}

const QMetaObject *DDM::DisplayManagerSession::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DDM::DisplayManagerSession::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM21DisplayManagerSessionE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DDM::DisplayManagerSession::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
