/****************************************************************************
** Meta object code from reading C++ file 'SocketServer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../SocketServer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SocketServer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3DDM12SocketServerE_t {};
} // unnamed namespace

template <> constexpr inline auto DDM::SocketServer::qt_create_metaobjectdata<qt_meta_tag_ZN3DDM12SocketServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DDM::SocketServer",
        "login",
        "",
        "QLocalSocket*",
        "socket",
        "user",
        "password",
        "Session",
        "session",
        "logout",
        "id",
        "lock",
        "unlock",
        "connected",
        "disconnected",
        "newConnection",
        "readyRead",
        "informationMessage",
        "message",
        "loginFailed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'login'
        QtMocHelpers::SignalData<void(QLocalSocket *, const QString &, const QString &, const Session &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'logout'
        QtMocHelpers::SignalData<void(QLocalSocket *, int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 10 },
        }}),
        // Signal 'lock'
        QtMocHelpers::SignalData<void(QLocalSocket *, int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 10 },
        }}),
        // Signal 'unlock'
        QtMocHelpers::SignalData<void(QLocalSocket *, const QString &, const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'connected'
        QtMocHelpers::SignalData<void(QLocalSocket *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void(QLocalSocket *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'newConnection'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'readyRead'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'informationMessage'
        QtMocHelpers::SlotData<void(QLocalSocket *, const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 18 },
        }}),
        // Slot 'loginFailed'
        QtMocHelpers::SlotData<void(QLocalSocket *, const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SocketServer, qt_meta_tag_ZN3DDM12SocketServerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DDM::SocketServer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM12SocketServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM12SocketServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3DDM12SocketServerE_t>.metaTypes,
    nullptr
} };

void DDM::SocketServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SocketServer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->login((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Session>>(_a[4]))); break;
        case 1: _t->logout((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->lock((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->unlock((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 4: _t->connected((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1]))); break;
        case 5: _t->disconnected((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1]))); break;
        case 6: _t->newConnection(); break;
        case 7: _t->readyRead(); break;
        case 8: _t->informationMessage((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->loginFailed((*reinterpret_cast<std::add_pointer_t<QLocalSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SocketServer::*)(QLocalSocket * , const QString & , const QString & , const Session & )>(_a, &SocketServer::login, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SocketServer::*)(QLocalSocket * , int )>(_a, &SocketServer::logout, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SocketServer::*)(QLocalSocket * , int )>(_a, &SocketServer::lock, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SocketServer::*)(QLocalSocket * , const QString & , const QString & )>(_a, &SocketServer::unlock, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SocketServer::*)(QLocalSocket * )>(_a, &SocketServer::connected, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SocketServer::*)(QLocalSocket * )>(_a, &SocketServer::disconnected, 5))
            return;
    }
}

const QMetaObject *DDM::SocketServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DDM::SocketServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3DDM12SocketServerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DDM::SocketServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void DDM::SocketServer::login(QLocalSocket * _t1, const QString & _t2, const QString & _t3, const Session & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 1
void DDM::SocketServer::logout(QLocalSocket * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void DDM::SocketServer::lock(QLocalSocket * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void DDM::SocketServer::unlock(QLocalSocket * _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void DDM::SocketServer::connected(QLocalSocket * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void DDM::SocketServer::disconnected(QLocalSocket * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
