/****************************************************************************
** Meta object code from reading C++ file 'Login1Session.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "Login1Session.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Login1Session.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN36OrgFreedesktopLogin1SessionInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopLogin1SessionInterface::qt_create_metaobjectdata<qt_meta_tag_ZN36OrgFreedesktopLogin1SessionInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopLogin1SessionInterface",
        "Lock",
        "",
        "PauseDevice",
        "in0",
        "in1",
        "in2",
        "ResumeDevice",
        "QDBusUnixFileDescriptor",
        "Unlock",
        "Activate",
        "QDBusPendingReply<>",
        "Kill",
        "requestLock",
        "PauseDeviceComplete",
        "ReleaseControl",
        "ReleaseDevice",
        "SetIdleHint",
        "TakeControl",
        "TakeDevice",
        "QDBusPendingReply<QDBusUnixFileDescriptor,bool>",
        "QDBusReply<QDBusUnixFileDescriptor>",
        "bool&",
        "out1",
        "Terminate",
        "requestUnlock",
        "Active",
        "Audit",
        "Class",
        "Desktop",
        "Display",
        "Id",
        "IdleHint",
        "IdleSinceHint",
        "IdleSinceHintMonotonic",
        "Leader",
        "Name",
        "Remote",
        "RemoteHost",
        "RemoteUser",
        "Scope",
        "Seat",
        "NamedSeatPath",
        "Service",
        "State",
        "TTY",
        "Timestamp",
        "TimestampMonotonic",
        "Type",
        "User",
        "NamedUserPath",
        "VTNr"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Lock'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'PauseDevice'
        QtMocHelpers::SignalData<void(uint, uint, const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 4 }, { QMetaType::UInt, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'ResumeDevice'
        QtMocHelpers::SignalData<void(uint, uint, const QDBusUnixFileDescriptor &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 4 }, { QMetaType::UInt, 5 }, { 0x80000000 | 8, 6 },
        }}),
        // Signal 'Unlock'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Activate'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(10, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Slot 'Kill'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, int)>(12, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::QString, 4 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'requestLock'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(13, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Slot 'PauseDeviceComplete'
        QtMocHelpers::SlotData<QDBusPendingReply<>(uint, uint)>(14, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::UInt, 4 }, { QMetaType::UInt, 5 },
        }}),
        // Slot 'ReleaseControl'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(15, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Slot 'ReleaseDevice'
        QtMocHelpers::SlotData<QDBusPendingReply<>(uint, uint)>(16, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::UInt, 4 }, { QMetaType::UInt, 5 },
        }}),
        // Slot 'SetIdleHint'
        QtMocHelpers::SlotData<QDBusPendingReply<>(bool)>(17, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'TakeControl'
        QtMocHelpers::SlotData<QDBusPendingReply<>(bool)>(18, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'TakeDevice'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusUnixFileDescriptor,bool>(uint, uint)>(19, 2, QMC::AccessPublic, 0x80000000 | 20, {{
            { QMetaType::UInt, 4 }, { QMetaType::UInt, 5 },
        }}),
        // Slot 'TakeDevice'
        QtMocHelpers::SlotData<QDBusReply<QDBusUnixFileDescriptor>(uint, uint, bool &)>(19, 2, QMC::AccessPublic, 0x80000000 | 21, {{
            { QMetaType::UInt, 4 }, { QMetaType::UInt, 5 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'Terminate'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(24, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Slot 'requestUnlock'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(25, 2, QMC::AccessPublic, 0x80000000 | 11),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Active'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'Audit'
        QtMocHelpers::PropertyData<uint>(27, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'Class'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Desktop'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Display'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Id'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'IdleHint'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'IdleSinceHint'
        QtMocHelpers::PropertyData<qulonglong>(33, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'IdleSinceHintMonotonic'
        QtMocHelpers::PropertyData<qulonglong>(34, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'Leader'
        QtMocHelpers::PropertyData<uint>(35, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'Name'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Remote'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'RemoteHost'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'RemoteUser'
        QtMocHelpers::PropertyData<QString>(39, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Scope'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Seat'
        QtMocHelpers::PropertyData<NamedSeatPath>(41, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Service'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'State'
        QtMocHelpers::PropertyData<QString>(44, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'TTY'
        QtMocHelpers::PropertyData<QString>(45, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Timestamp'
        QtMocHelpers::PropertyData<qulonglong>(46, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'TimestampMonotonic'
        QtMocHelpers::PropertyData<qulonglong>(47, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'Type'
        QtMocHelpers::PropertyData<QString>(48, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'User'
        QtMocHelpers::PropertyData<NamedUserPath>(49, 0x80000000 | 50, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'VTNr'
        QtMocHelpers::PropertyData<uint>(51, QMetaType::UInt, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopLogin1SessionInterface, qt_meta_tag_ZN36OrgFreedesktopLogin1SessionInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopLogin1SessionInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36OrgFreedesktopLogin1SessionInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36OrgFreedesktopLogin1SessionInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN36OrgFreedesktopLogin1SessionInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopLogin1SessionInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopLogin1SessionInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Lock(); break;
        case 1: _t->PauseDevice((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 2: _t->ResumeDevice((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[3]))); break;
        case 3: _t->Unlock(); break;
        case 4: { QDBusPendingReply<> _r = _t->Activate();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->Kill((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->requestLock();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->PauseDeviceComplete((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<> _r = _t->ReleaseControl();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->ReleaseDevice((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->SetIdleHint((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<> _r = _t->TakeControl((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<QDBusUnixFileDescriptor,bool> _r = _t->TakeDevice((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusUnixFileDescriptor,bool>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusReply<QDBusUnixFileDescriptor> _r = _t->TakeDevice((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusReply<QDBusUnixFileDescriptor>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<> _r = _t->Terminate();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<> _r = _t->requestUnlock();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1SessionInterface::*)()>(_a, &OrgFreedesktopLogin1SessionInterface::Lock, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1SessionInterface::*)(uint , uint , const QString & )>(_a, &OrgFreedesktopLogin1SessionInterface::PauseDevice, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1SessionInterface::*)(uint , uint , const QDBusUnixFileDescriptor & )>(_a, &OrgFreedesktopLogin1SessionInterface::ResumeDevice, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopLogin1SessionInterface::*)()>(_a, &OrgFreedesktopLogin1SessionInterface::Unlock, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->active(); break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->audit(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->className(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->desktop(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->display(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->id(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->idleHint(); break;
        case 7: *reinterpret_cast<qulonglong*>(_v) = _t->idleSinceHint(); break;
        case 8: *reinterpret_cast<qulonglong*>(_v) = _t->idleSinceHintMonotonic(); break;
        case 9: *reinterpret_cast<uint*>(_v) = _t->leader(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->remote(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->remoteHost(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->remoteUser(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->scope(); break;
        case 15: *reinterpret_cast<NamedSeatPath*>(_v) = _t->seat(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->service(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->state(); break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->tTY(); break;
        case 19: *reinterpret_cast<qulonglong*>(_v) = _t->timestamp(); break;
        case 20: *reinterpret_cast<qulonglong*>(_v) = _t->timestampMonotonic(); break;
        case 21: *reinterpret_cast<QString*>(_v) = _t->type(); break;
        case 22: *reinterpret_cast<NamedUserPath*>(_v) = _t->user(); break;
        case 23: *reinterpret_cast<uint*>(_v) = _t->vTNr(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgFreedesktopLogin1SessionInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopLogin1SessionInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36OrgFreedesktopLogin1SessionInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopLogin1SessionInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopLogin1SessionInterface::Lock()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OrgFreedesktopLogin1SessionInterface::PauseDevice(uint _t1, uint _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void OrgFreedesktopLogin1SessionInterface::ResumeDevice(uint _t1, uint _t2, const QDBusUnixFileDescriptor & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void OrgFreedesktopLogin1SessionInterface::Unlock()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
