/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.DisplayManager.Session.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "sessionadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class SessionAdaptor
 */

SessionAdaptor::SessionAdaptor(DDM::DisplayManagerSession *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

SessionAdaptor::~SessionAdaptor()
{
    // destructor
}

QDBusObjectPath SessionAdaptor::seat() const
{
    // get the value of property Seat
    return qvariant_cast< QDBusObjectPath >(parent()->property("Seat"));
}

QString SessionAdaptor::userName() const
{
    // get the value of property UserName
    return qvariant_cast< QString >(parent()->property("UserName"));
}

void SessionAdaptor::Lock()
{
    // handle method call org.freedesktop.DisplayManager.Session.Lock
    parent()->Lock();
}


#include "moc_sessionadaptor.cpp"
